import asyncio
import logging

import aiohttp
from aiohttp import web

import config


def method_url(method):
    if method in config.ALLOWED_APIS:
        return f'{config.BOT_API_SERVER}{config.API_TOKEN}/{method}'
    else:
        raise Exception(f"No such method: {method}")


class APIServerSession(aiohttp.ClientSession):
    """
    Bot access to telegram bot API

    """

    def __init__(self, *args, **kwargs):
        super(APIServerSession, self).__init__(*args, **kwargs)

        self.proxy = config.PROXY if config.PROXY else None
        asyncio.create_task(self.init_session())

    async def _request(self, *args, **kwargs):
        """
        Inject additional settings into aiohttp request API, such as proxy settings

        :param args:
        :param kwargs:
        :return:
        """
        return await super(APIServerSession, self)._request(
            *args, proxy=self.proxy, **kwargs
        )

    async def init_session(self):
        # Check if webhook is set already
        logging.info(f'Init new session to API server...')
        resp = await self.post(url=method_url('getWebhookInfo'))
        webhook_url = (await resp.json())['result']['url']
        logging.info(f'Current API webhook url is {webhook_url}')

        # Init already done
        if webhook_url == config.UPDATE_URL:
            logging.info(f'Using current webhook url')
            return

        logging.info(f'Setting new webhook url...')
        resp = await self.post(
            url=method_url('setWebhook'), json={'url': config.UPDATE_URL}
        )
        if resp.status != 200:
            raise Exception(
                f'Failed to init server session: {resp.status}, {resp.reason}'
            )
        webhook_url = (await resp.json())['result']['url']
        logging.warning(
            f'New session to API server created, current webhook url is {webhook_url}'
        )


class MessageHandleServer(web.Application):
    """
    Bot receives messages from telegram API server

    """

    def __init__(self, session, *args, **kwargs):
        super(MessageHandleServer, self).__init__()
        self.session = session
        self.add_routes(
            [
                web.post(config.API_TOKEN, self.resp),
                web.post(f'{config.API_TOKEN}/', self.resp),
            ]
        )

    def run(self):
        if config.USE_REVERSE_PROXY:
            host, port = config.LISTEN_ON, config.LISTEN_PORT
        else:
            host, port = config.YOUR_SERVER, config.PORT

        web.run_app(self, host=str(host), port=int(port))

    async def resp(self, request: web.Request, *args, **kwargs) -> web.Response:
        pass
